; Contents: Wart Boss + Bubble Sprite
; They are seperated by sprite states - they take the same sprite slot
; for custom sprites upon insertion with sprite tool, but in-game they
; happen to be different sprites.
;
; Sprite mode 10 is therefore most recommended, to be able to process
; as many sprites as possible on-screen.
; Usage with edit1754's No More Sprite Tile Limits patch is also an option.

;----------------;
; Various defs   ;
;----------------;

!VegSprNum = $01 	; IMPORTANT! Custom sprite number you insert veggieshrug.asm as.
!BubbleProp = $2D 	; YXPPCCCT byte. Affects palette of bubble - not of Wart.
!HitPoints = $06 	; Amount of hits until Wart dies.
!DoorYCoord = $0170	; Y coordinate of the door that appears.
!DoorXCoord = $0280	; X coordinate of the door that appears.
!SongToPlay = $0B	; Song to play when Wart dies.

;----------------------------;
;        INIT ROUTINE        ;
;----------------------------;

print "INIT ",pc
JSR SUB_HORZ_POS
TYA
STA $157C,x
RTL				;  Terminate.

;----------------------------;
;       MAIN ROUTINE         ;
;----------------------------;

print "MAIN ",pc
PHB             ; Wrapper.
PHK
PLB
JSR MainSpr
PLB
RTL

;-----------;

MainSpr:
LDA $1528,x
PHX
ASL A
TAX
JMP (SpritePtr,x)

SpritePtr:
dw Wart
dw Bubble

Wart:
PLX
JSR DrawGFX
LDA $14C8,x
CMP #$08
BNE Return
LDA $9D
BNE Return
LDA $14D4,x
XBA
LDA $D8,x
REP #$20
CMP #$01D0
SEP #$20
BCS GenerateDoor
JSR GeneralMovement
JSR ThrowingBubbles
JSR MarSprInt
JSR SprSprInt
JSR HandlePain

Return:
RTS

Bubble:
PLX
JSR BubbleGFX
LDA $14C8,x
CMP #$08
BNE Return
LDA $9D
BNE Return
DEC $AA,x
LDA $13
AND #$01
BNE Gravity
DEC $AA,x

Gravity:
JSL $01802A
JSR MarSprInt
LDA $14D4,x
XBA
LDA $D8,x
REP #$20
CMP #$01B0
SEP #$20
BCS Terminate
RTS

Terminate:
LDA #$00
STA $9E,x
STA $14C8,x
STA $7FAB9E,x
STA $7FAB10,x
RTS

;----------------------;
; Terminate - Gen Door ;
;----------------------;

GenerateDoor:
PHP
REP #$30
LDA #!DoorYCoord
STA $98
LDA #!DoorXCoord
STA $9A
LDA #$001F
STA $03                 ;  |
JSR SUBL_SET_MAP16      ;  |
LDA $98
CLC
ADC #$0010
STA $98
LDA #$0020
STA $03                 ;  |
JSR SUBL_SET_MAP16      ;  |
PLP                     ; / 
LDA $E4,x
PHA
LDA $14E0,x
PHA
LDA $D8,x
PHA
LDA $14D4,x
PHA
LDA.b #!DoorXCoord
STA $E4,x
LDA.b #!DoorXCoord/256
STA $14E0,x
LDA.b #!DoorYCoord
STA $D8,x
LDA.b #!DoorYCoord/256
STA $14D4,x
TXY
JSR GenerateSmoke
LDA $D8,x
CLC
ADC #$10
STA $D8,x
LDA $14D4,x
ADC #$00
STA $14D4,x
JSR GenerateSmoke
PLA
STA $14D4,x
PLA
STA $D8,x
PLA
STA $14E0,x
PLA
STA $E4,x
LDA #!SongToPlay
STA $1DFB
JMP Terminate

;-----------------;
; Hurt/Die - Wart ;
;-----------------;

HandlePain:
LDA $151C,x
CMP #!HitPoints
BCC NoKill
LDA #$04
STA $1602,x
LDA $154C,x
BNE TimerNotDone
LDA $1540,x
BNE DeathTimerRunning
LDA #$7F
STA $1540,x
LDA #$E0
STA $AA,x
LDA #$08
STA $B6,x
LDA $1686,x
ORA #$80
STA $1686,x

DeathTimerRunning:
DEC $AA,x
DEC $AA,x
JSL $01802A
LDA $1540,x
AND #$1F
BNE TimerNotDone
LDA #$20
STA $1DFC
TXY
JSR GenerateSmoke

TimerNotDone:
RTS

PainFrames:
db $02,$04,$04,$04,$02,$04

NoKill:
LDA $154C,x
BEQ NoFlashingHurt
CMP #$01
BEQ ResetAll
LSR A
LSR A
LSR A
LSR A
TAY
LDA PainFrames,y
STA $1602,x
RTS

ResetAll:
LDA #$FF
STA $1558,x
STZ $1504,x

NoFlashingHurt:
RTS

;----------------;
; Walking - Wart ;
;----------------;

WalkingImages:
db $00,$01,$00,$01

WalkingSpeed:
db $F8,$08

GeneralMovement:
LDA $154C,x
BNE NoMovingHurt
LDA $151C,x
CMP #!HitPoints
BCS NoFlashingHurt
LDA $C2,x
AND #$01
BNE NoMovement
LDA $1564,x
BNE ReturnMoving
LDA #$56
STA $1564,x
INC $C2,x
RTS

NoMovingHurt:
INC $1564,x
RTS

ReturnMoving:
STZ $00
LDA $1504,x
BNE NotBurping
LDA $1558,x
CMP #$56
BCS NotBurping
LDA #$02
STA $00

NotBurping:
LDA $1564,x
AND #$30
LSR A
LSR A
LSR A
LSR A
TAY
LDA WalkingImages,y
ORA $00
STA $1602,x
LDA $C2,x
AND #$02
LSR A
TAY
LDA WalkingSpeed,y
STA $B6,x
JSL $01802A
RTS

NoMovement:
STZ $1602,x
LDA $1564,x
BNE ReallyDont
LDA #$40
STA $1564,x
INC $C2,x
LDA $C2,x
CMP #$04
BCC ReallyDont
STZ $C2,x

ReallyDont:
RTS

;---------------------------;
; Burping up Bubbles - Wart ;
;---------------------------;

ThrowingBubbles:
LDA $154C,x
BNE NoThrowing
LDA $151C,x
CMP #!HitPoints
BCS NoThrowing
LDA $1504,x
BNE DoubleTheTime
LDA $1558,x
BNE ThrowBubbles

SetAwfulTimer:
LDA #$96
STA $1558,x
INC $1504,x
LDA $1504,x
CMP #$02
BCC NoThrowing
STZ $1504,x
BRA NoThrowing

DoubleTheTime:
LDA $1558,x
BEQ SetAwfulTimer
RTS

ThrowBubbles:
CMP #$56
BEQ SetRandomPoint
BCS NoThrowing
LDA $C2,x		; \ So that the burping image is shown.
AND #$01		;  | Even if Wart is standing still.
BEQ ImgProof		; /
LDA #$02
STA $1602,x

ImgProof:
LDA $1558,x
AND #$0F
CMP #$09
BNE NoThrowing
JSR BubblesOneByOne

NoThrowing:
RTS

SetRandomPoint:
JSL $01ACF9
AND #$07
SEC
SBC #$03
STA $1510,x
RTS

;--------------------------;
; Mario <-> Sprite interact;
;--------------------------;

MarSprInt:
JSL $03B664
JSR GetSpriteClipping
JSL $03B72B
BCC ReturnInteraction
JSL $00F5B7

ReturnInteraction:
RTS

;---------------------------;
; Sprite <-> Sprite interact;
;---------------------------;

SprSprInt:
LDA $1504,x
BNE ReturnInteraction
LDA $1558,x
CMP #$56
BCS ReturnInteraction
LDY #$0B

SprIntLoop:
LDA $14C8,y
CMP #$08
BCC DecreaseIndxSpr
PHX
TYX
LDA $7FAB9E,x
PLX
CMP #!VegSprNum
BNE DecreaseIndxSpr
LDA $1528,y
BEQ DecreaseIndxSpr
PHX
TYX
JSR GetVegClipping
PLX
JSR GetMouthClipping
JSL $03B72B
BCC DecreaseIndxSpr
INC $151C,x
LDA $151C,x
CMP #!HitPoints
BNE LessTimeForFlash
LDA #$8F
BRA FlashingPain

LessTimeForFlash:
LDA #$5F

FlashingPain:
STA $154C,x
LDA #$28
STA $1DFC
JSR GenerateSmoke
PHX
TYX
LDA #$00
STA $9E,x
STA $14C8,x
STA $7FAB9E,x
STA $7FAB10,x
PLX
RTS

DecreaseIndxSpr:
DEY
BPL SprIntLoop
RTS

;--------------------------;
; Sprite Clipping Routine  ;
; Mostly borrowed from SMW ;
;--------------------------;

GetVegClipping:
LDA $E4,x
STA $00
LDA $14E0,x
STA $08
LDA #$0D                ; Width of sprite clipping
STA $02
LDA $D8,x
STA $01
LDA $14D4,x
STA $09
LDA #$0D                ; Height of sprite clipping
STA $03
RTS

GetMouthClipping:
LDA $E4,x
SEC
SBC #$0A
STA $04
LDA $14E0,x
SBC #$00
STA $0A
LDA #$14                ; Width of sprite clipping
STA $06
LDA $D8,x
SEC
SBC #$14
STA $05
LDA $14D4,x
SBC #$00
STA $0B
LDA #$0C                ; Height of sprite clipping
STA $07
RTS

XDispClip:
db $16,$00

WidthClip:
db $24,$0B

HeightClip:
db $2C,$0B

GetSpriteClipping:
PHY
LDY $1528,x
LDA $E4,x
SEC
SBC XDispClip,y
STA $04
LDA $14E0,x
SBC #$00
STA $0A
LDA WidthClip,y                ; Width of sprite clipping
STA $06
LDA $D8,x
SEC
SBC XDispClip,y
STA $05
LDA $14D4,x
SBC #$00
STA $0B
LDA HeightClip,y                ; Height of sprite clipping
STA $07
PLY

RETURN67:
RTS

;-------------------;
; Spitting Routine. ;
;-------------------;

XDispLo:
db $04,$0C

BubbleSpeedX:
db $08,$14,$1C,$26,$30

BubblesOneByOne:
LDA $15D0,x
BNE RETURN67
                    
JSL $02A9DE             ; \ get an index to an unused sprite slot, return if all slots full
BMI RETURN67            ; / after: Y has index of sprite being generated
LDA #$25                ; \ sound effect
STA $1DF9 
LDA #$08                ; \ set sprite status for new sprite
STA $14C8,y             ; /
PHX
LDA $7FAB9E,x
TYX
STA $7FAB9E,x
PLX

LDA $E4,x               ; \ set x position for new sprite
PHY
LDY $157C,x
SEC
SBC XDispLo,y
PLY
STA $00E4,y
LDA $14E0,x
SBC #$00
STA $14E0,y

LDA $D8,x               ; \ set y position for new sprite
SEC
SBC #$10
STA $00D8,y             ;  |
LDA $14D4,x             ;  |
SBC #$00
STA $14D4,y             ; /

PHX                     ; \ before: X must have index of sprite being generated
TYX                     ;  | routine clears *all* old sprite values...
JSL $07F7D2             ;  | ...and loads in new values for the 6 main sprite tables
JSL $0187A7             ;  get table values for custom sprite
LDA #$88
STA $7FAB10,x
PLX                       ; / 

LDA #$01
STA $1528,y
LDA $1686,y
ORA #$80
STA $1686,y
LDA $1510,x
STA $00
LDA $1558,x
AND #$70
LSR A
LSR A
LSR A
LSR A
PHY
TAY
LDA BubbleSpeedX,y
CLC
ADC $00
LDY $157C,x
BEQ NoXOR
EOR #$FF
INC A

NoXOR:
PLY
STA $00B6,y
LDA #$D8
STA $00AA,y
RTS                     ; return

;------------------;
; Smoke Generation ;
;------------------;

GenerateSmoke:
PHX
LDX #$03

SmokeLooping:
LDA $17C0,x
BEQ BrkSmokeLoop
DEX
BPL SmokeLooping
LDX #$03

BrkSmokeLoop:
LDA #$01
STA $17C0,x
LDA $00D8,y
STA $17C4,x
LDA $00E4,y
STA $17C8,x
LDA #$1B
STA $17CC,x
PLX
RTS

;----------------------------------------------------------------------------------------------;
;                  GRAPHICS DATA   ;
;----------------------------------------------------------------------------------------------;

WartXDisp:
db $E8,$F8,$08,$E8,$F8,$08,$E8,$F8,$08
db $08,$F8,$E8,$08,$F8,$E8,$08,$F8,$E8

WartYDisp:
db $E0,$E0,$E0,$F0,$F0,$F0,$00,$00,$00

WartTileMap:
db $C0,$C2,$C4,$C6,$C8,$CA,$CC,$CE,$E0
db $C0,$C2,$C4,$C6,$C8,$CA,$E2,$E4,$E0
db $88,$8A,$8C,$EC,$EE,$CA,$CC,$CE,$E0
db $88,$8A,$8C,$EC,$EE,$CA,$E2,$E4,$E0
db $E6,$E8,$EA,$EC,$EE,$CA,$E2,$E4,$E0

DrawGFX:
JSR GET_DRAW_INFO	; after: Y = index to sprite tile map ($300)
LDA $1602,x		; \ Multiply GFX pointer by 9.
ASL A			;  |
ASL A			;  |
ASL A			;  |
CLC			;  |
ADC $1602,x		;  | 
STA $04			; / Index to images.
LDA $157C,x		; \ Get direction.
EOR #$01		;  | Correction
AND #$01		;  | Only one bit is needed.
PHA			;  | Stack.
ASL A			;  | Multiply by 9.
ASL A			;  |
ASL A			;  |
CLC			;  |
ADC $01,s		;  |
STA $03			;  | Index to XDisp - either the first 9 or the latter 9 values.
PLA			;  |
CLC			;  |
ROR A			;  |
ROR A			;  |
ROR A			;  |
STA $02			; / $02 is now fit to be OR'd to form the value in $0303,y - as it is either #$00 or #$40 (X-flip)
LDA $15F6,x
PHY
LDY $154C,x
BEQ NotHurt
LDA $14
AND #$1C
LSR A
ORA #$01

NotHurt:
PLY
STA $05
PHX
LDX #$08

GFXLoop:            
PHX
TXA
CLC
ADC $03
TAX
LDA $00
CLC
ADC WartXDisp,x
STA $0300,y             ; /
PLX

LDA $01                 ;  | tile y position = sprite y location ($01) + tile displacement
CLC
ADC WartYDisp,x
STA $0301,y             ; /

PHX
TXA
CLC
ADC $04
TAX
LDA WartTileMap,x
PLX
STA $0302,y             ; / 

LDA $05
ORA $64
ORA $02
STA $0303,y             ; / store tile properties

PHX
TYA 			; \ get index to sprite property map ($460)...
LSR A                   ;  |    ...we use the sprite OAM index...
LSR A                   ;  |    ...and divide by 4 because a 16x16 tile is 4 8x8 tiles
TAX 			;  |
LDA #$02		;  |
STA $0460,x             ; /
PLX

INY
INY
INY
INY
DEX
BPL GFXLoop

PLX 			; pull, X = sprite index
LDY #$FF                ; \ why FF? (460 &= 2) 8x8 tiles maintained
LDA #$08                ;  | A = number of tiles drawn - 1
JSL $01B7B3             ; / don't draw if offscreen

RETURN1337:
RTS 			; return

BubbleImg:
db $A0,$A2

BubbleGFX:
JSR GET_DRAW_INFO	; after: Y = index to sprite tile map ($300)
LDA $157C,x
AND #$01
CLC
ROR A
ROR A
ROR A
STA $02
LDA $14
AND #$08
LSR A
LSR A
LSR A
STA $03

LDA $00
STA $0300,y
LDA $01
STA $0301,y
PHY
LDY $03
LDA BubbleImg,y
PLY
STA $0302,y
LDA #!BubbleProp
ORA $02
STA $0303,y

LDY #$02                ; \ why FF? (460 &= 2) 8x8 tiles maintained
LDA #$00                ;  | A = number of tiles drawn - 1
JSL $01B7B3             ; / don't draw if offscreen
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; routines below can be shared by all sprites.  they are ripped from original
; SMW and poorly documented
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_HORZ_POS:
LDY #$00
LDA $94
SEC
SBC $E4,x
STA $0E
LDA $95
SBC $14E0,x
STA $0F
BPL LABEL16
INY
LABEL16:
RTS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;org $03B75C

TABLE1:
db $0C,$1C

TABLE2:
db $01,$02

GET_DRAW_INFO:
STZ $186C,x             ; reset sprite offscreen flag, vertical
STZ $15A0,x             ; reset sprite offscreen flag, horizontal
LDA $E4,x               ; \
CMP $1A                 ;  | set horizontal offscreen if necessary
LDA $14E0,x             ;  |
SBC $1B                 ;  |
BEQ ON_SCREEN_X         ;  |
INC $15A0,x             ; /

ON_SCREEN_X:
LDA $14E0,x             ; \
XBA 			;  |
LDA $E4,x               ;  |
REP #$20                ;  |
SEC 			;  |
SBC $1A                 ;  | mark sprite invalid if far enough off screen
CLC 			;  |
ADC #$0040            	;  |
CMP #$0180            	;  |
SEP #$20                ;  |
ROL A                   ;  |
AND #$01                ;  |
STA $15C4,x             ; / 
BNE INVALID             ; 

LDY #$00                ; \ set up loop:
LDA $1662,x             ;  | 
AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
INY 			; / 

ON_SCREEN_LOOP:
LDA $D8,x               ; \ 
CLC 			;  | set vertical offscreen if necessary
ADC TABLE1,y            ;  |
PHP 			;  |
CMP $1C                 ;  | (vert screen boundry)
ROL $00                 ;  |
PLP 			;  |
LDA $14D4,x             ;  | 
ADC #$00                ;  |
LSR $00                 ;  |
SBC $1D                 ;  |
BEQ ON_SCREEN_Y         ;  |
LDA $186C,x             ;  | (vert offscreen)
ORA TABLE2,y            ;  |
STA $186C,x             ;  |

ON_SCREEN_Y:
DEY 			;  |
BPL ON_SCREEN_LOOP      ; /

LDY $15EA,x             ; get offset to sprite OAM
LDA $E4,x               ; \ 
SEC 			;  | 
SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
STA $00                 ; / 
LDA $D8,x               ; \ 
SEC 			;  | 
SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
STA $01                 ; / 
RTS 			; return

INVALID:
PLA 			; \ return from *main gfx routine* subroutine...
PLA 			;  |    ...(not just this subroutine)
RTS 			; /


;-----------------;
; Change M16 code ;
;-----------------;

SUBL_SET_MAP16:
PHP
REP #$30 
PHY
PHX
TAX
LDA $03
PHA
JSR SUB_8034
PLA
STA $03
PLX
PLY
PLP
RTS

RETURN18:
PLX
PLB
PLP
RTS

SUB_8034:
PHP
SEP #$20
PHB
LDA #$00
PHA
PLB
REP #$30
PHX
LDA $9A
STA $0C
LDA $98
STA $0E
LDA #$0000
SEP #$20
LDA $5B
STA $09
LDA $1933
BEQ NO_SHIFT
LSR $09

NO_SHIFT:
LDY $0E
LDA $09
AND #$01
BEQ HORIZ
LDA $9B
STA $00
LDA $99
STA $9B
LDA $00  
STA $99
LDY $0C

HORIZ:
CPY #$0200
BCS RETURN18
LDA $1933
ASL A
TAX
LDA $BEA8,x
STA $65
LDA $BEA9,x
STA $66
STZ $67
LDA $1925
ASL A
TAY
LDA ($65),y
STA $04
INY
LDA ($65),y
STA $05
STZ $06
LDA $9B
STA $07
ASL A
CLC
ADC $07
TAY
LDA ($04),y
STA $6B
STA $6E
INY
LDA ($04),y
STA $6C
STA $6F
LDA #$7E
STA $6D
INC A
STA $70
LDA $09
AND #$01
BEQ NO_AND
LDA $99
LSR A
LDA $9B 
AND #$01
BRA LABEL52

NO_AND:
LDA $9B
LSR A
LDA $99

LABEL52:
ROL A
ASL A
ASL A
ORA #$20
STA $04
CPX #$0000
BEQ NO_ADD
CLC
ADC #$10 
STA $04

NO_ADD:
LDA $98
AND #$F0
CLC
ASL A
ROL A
STA $05
ROL A
AND #$03
ORA $04
STA $06
LDA $9A
AND #$F0
LSR A
LSR A
LSR A
STA $04
LDA $05
AND #$C0
ORA $04
STA $07
REP #$20
LDA $09
AND #$0001
BNE LABEL51
LDA $1A
SEC
SBC #$0080
TAX
LDY $1C
LDA $1933
BEQ LABEL50
LDX $1E
LDA $20
SEC
SBC #$0080
TAY
BRA LABEL50

LABEL51:
LDX $1A
LDA $1C
SEC
SBC #$0080
TAY
LDA $1933
BEQ LABEL50
LDA $1E
SEC
SBC #$0080
TAX  
LDY $20

LABEL50:
STX $08
STY $0A
LDA $98
AND #$01F0
STA $04
LDA $9A
LSR A
LSR A
LSR A
LSR A
AND #$000F
ORA $04
TAY
PLA
SEP #$20
STA [$6B],y
XBA
STA [$6E],y
XBA
REP #$20
ASL A
TAY
PHK
PEA.w MAP16_RETURN-1
PEA $804C
JMP $00C0FB

MAP16_RETURN:
PLB
PLP
RTS